# Import packages
import sqlite3
import requests
from bs4 import BeautifulSoup as b
from tabulate import tabulate


## NOTE : 
## 1. DO NOT CHANGE THE NAME OF ANY FUNCTION OR ANY ARGUMENT OR CLASS NAME. 
## 2. DO NOT CHANGE ANYTHING IN MAIN FUCNTION.
## 3. WRITE YOUR CODE INSIDE ### START CODE HERE ### and ### END CODE HERE ### ONLY
## 4. ANY DEVIATION IN THE NAMING CONVENTION, THE AUTOGRADER WILL MARK ZERO.

class CSE_Courses:

    def __init__(self):
        """
            Create the database CSE_Courses_DB and a table CSE_Courses
        """
        ### START CODE HERE ###
        self.conn = sqlite3.connect('CSE_Courses_DB.db', timeout=10)
        c = self.conn.cursor()

        c.execute(''' SELECT count(name) FROM sqlite_master WHERE type='table' AND name='CSE_Courses' ''')

        if not c.fetchone()[0]:
            c.execute('CREATE TABLE CSE_Courses (Course_Code text, Name text, Instructor text)')
        else:
            c.execute('delete from CSE_Courses')

        ### END CODE HERE ###

    def get_courses(self, url):
        """
            This method should scrap all the courses from the url and 
            returns list of lists where each inner list has [course code,name,instructor]
            of a course. 

            Arguments:
            url : url from which courses have to be scraped, string

            Returns:
            courses : courses scraped from the url : list of lists

        """
        assert (isinstance(url, str))
        ### START CODE HERE ###
        page = requests.get(url)
        soup = b(page.content, 'html.parser')
        # soup = BeautifulSoup(response.text,"html.parser")
        table = soup.findAll('table', {"class": "greytable"})[0]
        tr = table.findAll(['tr'])[1:44]
        courses = []
        for data in tr:
            td = data.find_all('td', {"class": "greybodytd"})
            courses.append([i.text.replace('\n', '') and i.text.strip() for i in td])
        ### END CODE HERE ###
        assert (isinstance(courses, list) and isinstance(courses[0], list))
        return courses

    def insert_data(self, courses):
        """
            This method will insert all the courses in the table CSE_Courses
            If you rerun your program, it will insert all the courses again 
            in the table, so if you want to delete the previously added course, 
            you may call delete_data() method from the main function.

            Arguments:
            courses : courses scrapped from the url : list of lists

            Returns:
            Nothing
            """
        assert (isinstance(courses, list) and isinstance(courses[0], list))
        ### START CODE HERE ###
        # conn = sqlite3.connect('CSE_Courses_DB.db', timeout=10)
        c = self.conn.cursor()
        l = len(courses)
        i = 0
        count = 0
        for i in range(l):
            code = courses[i][count]
            # print(code)
            name = courses[i][count + 1]
            # print(name)
            inst = courses[i][count + 2]
            # print(inst)
            c.execute(''' INSERT INTO CSE_Courses(Course_Code, Name, Instructor) VALUES(?,?,?)''', (code, name, inst))
            i = i + 1
            self.conn.commit()
        ### END CODE HERE ###

    def print_data(self):
        """
        This method will print all the courses present in the table CSE_Courses
        in a tabular format [you may use tabulate or PrettyTable, 
        feel free to use any of your choice, but the output should be in tabular formmat]

        Arguments:
        Nothing

        Returns:
        Nothing
        """

        ### START CODE HERE ###

        # conn = sqlite3.connect('CSE_Courses_DB.db', timeout=10)
        c = self.conn.cursor()
        c.execute('SELECT * from CSE_Courses')
        h_list = [member[0] for member in c.description]
        # print(h_list)
        print(tabulate(c, headers=h_list, tablefmt="fancy_grid"))
        ### END CODE HERE ###
        return None

    def delete_data(self, course_code=None):
        """
        This method will delete the particular course if course_code is passed,
        otherwise if course code is not passed, it will delete all the records 
        from the table CSE_Courses

            Arguments:
            course_code : course code which has to be deleted from the table CSE_Courses : string

            Returns:
            Nothingassert(isinstance(url,str))
            ### START CODE HERE ###
            page = requests.get(url)
            print(page)
            ### END CODE HERE ###
            assert(isinstance(courses,list) and isinstance(courses[0],list))
            return None

        """
        if course_code:
            assert (isinstance(course_code, str))
        ### START CODE HERE ###
        # conn = sqlite3.connect('CSE_Courses_DB.db', timeout=10)
        c = self.conn.cursor()
        del_all = 'delete from CSE_Courses'
        del_one = 'delete from CSE_Courses where Course_Code=?'
        # c.execute('delete from CSE_Courses where Course_Code=\"CS 101\"')
        if course_code == None:
            c.execute(del_all)
            # print( "all been deleted from CSE_Courses")
            self.conn.commit()

        else:
            c.execute(del_one, (course_code,))
            # print("one record deleted from CSE_Courses")
            self.conn.commit()

        ### END CODE HERE ###


if __name__ == "__main__":
    url = "https://www.cse.iitb.ac.in/archive/page136"
    cse = CSE_Courses()
    courses = cse.get_courses(url)
    cse.insert_data(courses)
    cse.print_data()
    #cse.delete_data()
    #cse.print_data()
